#-------------------------------------------------------------------------------
# Return the first element of a list
# - CAR(FIRST ${list})
#-------------------------------------------------------------------------------
FUNCTION(CAR var)
  SET(${var} ${ARGV1} PARENT_SCOPE)
ENDFUNCTION(CAR)

#-------------------------------------------------------------------------------
# Return the given list minus the first element
# - CDR(REST ${list})
#-------------------------------------------------------------------------------
FUNCTION(CDR var first)
  SET(${var} ${ARGN} PARENT_SCOPE)
ENDFUNCTION(CDR)

#-------------------------------------------------------------------------------
# add subdirectories to the project
# - SET(LIBS "src/lib1" "src/lib2")
# - ADD_SUBDIRECTORIES(${LIBS})
#-------------------------------------------------------------------------------

FUNCTION(ADD_SUBDIRECTORIES)
  MESSAGE(STATUS "Adding subdirectories")
  FOREACH(subdir ${ARGN})
    MESSAGE(STATUS "+ ${subdir}")
    ADD_SUBDIRECTORY("${subdir}" "${CMAKE_BINARY_DIR}/CMakeDirs/${subdir}")
  ENDFOREACH(subdir ${ARGN})
ENDFUNCTION(ADD_SUBDIRECTORIES)

#-------------------------------------------------------------------------------
# collect values of variable PUBLIC_HEADERS from given subdirectories LIB_DIRS
# - GET_PROPERTIES(PUBLIC_HEADERS FROM ${LIB_DIRS})
#-------------------------------------------------------------------------------

FUNCTION(GET_PROPERTIES)
  CMAKE_PARSE_ARGUMENTS(GET "" "" "FROM" ${ARGN})
  FOREACH(variable ${GET_UNPARSED_ARGUMENTS})
    SET(properties)
    FOREACH(dir ${GET_FROM})
      GET_DIRECTORY_PROPERTY(property DIRECTORY "${dir}" DEFINITION ${variable})
      LIST(APPEND properties ${property})
    ENDFOREACH(dir ${dirs})
    SET(${variable} ${properties} PARENT_SCOPE)
  ENDFOREACH(variable ${GET_ARGN})
ENDFUNCTION(GET_PROPERTIES)

#-------------------------------------------------------------------------------
# Figure out debian architecture
#-------------------------------------------------------------------------------

FUNCTION(GET_DEBIAN_ARCHITECTURE arch)
  SET(dpkgarch)
  FIND_PROGRAM(DPKG_CMD dpkg)
  IF(NOT DPKG_CMD)
    MESSAGE(STATUS "Can not find dpkg in your path, default to i386.")
    SET(${arch} i386 PARENT_SCOPE)
  ENDIF(NOT DPKG_CMD)
  EXECUTE_PROCESS(COMMAND "${DPKG_CMD}" --print-architecture
    OUTPUT_VARIABLE dpkgarch
    OUTPUT_STRIP_TRAILING_WHITESPACE
    )
  SET(${arch} ${dpkgarch} PARENT_SCOPE)
ENDFUNCTION(GET_DEBIAN_ARCHITECTURE)


#-------------------------------------------------------------------------------
# Record CPACK_INSTALL_CMAKE_PROJECTS quadruple for later use with ADD_PACKAGE
#-------------------------------------------------------------------------------

FUNCTION(ADD_TO_PACKAGE)
  CMAKE_PARSE_ARGUMENTS(PKG "" "NAME;COMPONENT" "" ${ARGN})
  SET(${PKG_NAME}_INSTALL ${${PKG_NAME}_INSTALL} "${CMAKE_BINARY_DIR};${PROJECT_NAME};${PKG_COMPONENT};/" PARENT_SCOPE)
ENDFUNCTION(ADD_TO_PACKAGE)

ADD_CUSTOM_TARGET(packages)

#-------------------------------------------------------------------------------
# Generate cpack configuration file and target to build a package
#-------------------------------------------------------------------------------
FUNCTION(ADD_PACKAGE)
  CMAKE_PARSE_ARGUMENTS(PKG ""
    "NAME;SUMMARY;DESCRIPTION;DEPENDS;RECOMMENDS;SUGGESTS;PREDEPENDS;ENHANCES;BREAKS;CONFLICTS;PROVIDES;REPLACES;SECTION;PRIORITY;VERSION;REVISION;VENDOR;CONTACT;PREINST;PRERM;POSTINST;POSTRM" 
    "SCAN" 
    ${ARGN})
  SET(config "${CMAKE_BINARY_DIR}/release/debian-package-${PKG_NAME}.cmake")
  SET(target "package_${PKG_NAME}")
  GET_PROPERTIES(${PKG_NAME}_INSTALL FROM ${PKG_SCAN})
  SET(PKG_INSTALL_CMAKE_PROJECTS "${${PKG_NAME}_INSTALL}")
  GET_DEBIAN_ARCHITECTURE(PKG_ARCHITECTURE)
  CONFIGURE_FILE("${CMAKE_SOURCE_DIR}/debian-package.in" "${config}" @ONLY)
  ADD_CUSTOM_TARGET(${target} COMMAND "${CMAKE_CPACK_COMMAND}" --config "${config}" WORKING_DIRECTORY "${CMAKE_BINARY_DIR}/release")
  ADD_DEPENDENCIES(packages ${target})  
ENDFUNCTION(ADD_PACKAGE)
